@@ -1,26 +1,23 @@
 from beir.datasets.data_loader import GenericDataLoader
-from sentence_transformers import SentenceTransformer, losses, models
-from .toolkit import qgen, NegativeMiner, MarginDistillationLoss, GenerativePseudoLabelingDataset, PseudoLabeler, evaluate, resize, mnrl
+from .toolkit import qgen, NegativeMiner, MarginDistillationLoss, GenerativePseudoLabelingDataset, PseudoLabeler, evaluate, resize, load_sbert, set_logger_format
+from sentence_transformers import SentenceTransformer
 from torch.utils.data import DataLoader
-
 import os
 import logging
-
 import argparse
 from typing import List
+# import crash_ipdb
+
+
+set_logger_format()
+logger = logging.getLogger('gpl.train')  # Here we do not use __name__ to have unified logger name, no matter whether we are using `python -m` or `import gpl; gpl.train`
 
-logger = logging.getLogger(__name__)
-logging.basicConfig(
-    format='%(asctime)s - %(message)s',
-    datefmt='%Y-%m-%d %H:%M:%S',
-    level=logging.INFO
-)
 
 def train(
     path_to_generated_data: str,
-    evaluation_data: str,
     output_dir: str,
     do_evaluation: str = False,
+    evaluation_data: str = None,
     evaluation_output: str = 'output',
     qgen_prefix: str = 'qgen',
     base_ckpt: str = 'distilbert-base-uncased',
@@ -28,7 +25,7 @@ def train(
     cross_encoder: str = 'cross-encoder/ms-marco-MiniLM-L-6-v2',
     batch_size_gpl: int = 32,
     batch_size_generation: int = 32,
-    pooling: str = 'mean',
+    pooling: str = None,
     max_seq_length: int = 350,
     new_size: int = None,
     queries_per_passage: int = 3,
@@ -38,7 +35,7 @@ def train(
     negatives_per_query: int = 50
 ):     
     #### Assertions ####
-    assert pooling in ['mean', 'cls', 'max']
+    assert pooling in [None, 'mean', 'cls', 'max']
     if do_evaluation:
         assert evaluation_data is not None
         assert evaluation_output is not None
@@ -50,6 +47,7 @@ def train(
     if new_size is not None:
         assert new_size * queries_per_passage >= batch_size_gpl
 
+
     #### Make sure there is a `corpus.jsonl` file. It should be under either `path_to_generated_data` or `evaluation_data`` ####
     #### Also resize the corpus for efficient training if required  ####
     os.makedirs(path_to_generated_data, exist_ok=True)
@@ -62,6 +60,7 @@ def train(
             corpus_path = os.path.join(evaluation_data, 'corpus.jsonl')
             os.system(f'cp {corpus_path} {path_to_generated_data}')
     
+
     #### Synthetic query generation ####
     #### This will be skipped if there is an existing `gen-queries.jsonl`file under `path_to_generated_data` ####
     if f'{qgen_prefix}-qrels' in os.listdir(path_to_generated_data) and f'{qgen_prefix}-queries.jsonl' in os.listdir(path_to_generated_data):
@@ -73,6 +72,7 @@ def train(
         qgen(path_to_generated_data, path_to_generated_data, generator_name_or_path=generator, ques_per_passage=queries_per_passage, bsz=batch_size_generation, qgen_prefix=qgen_prefix)
         corpus, gen_queries, gen_qrels = GenericDataLoader(path_to_generated_data, prefix=qgen_prefix).load(split="train")
 
+
     #### Hard-negative mining ####
     #### This will be skipped if there is an existing `hard-negatives.jsonl` file under `path_to_generated_data` ####
     if 'hard-negatives.jsonl' in os.listdir(path_to_generated_data):
@@ -82,6 +82,7 @@ def train(
         miner = NegativeMiner(path_to_generated_data, qgen_prefix, retrievers=retrievers, nneg=negatives_per_query)
         miner.run()
 
+
     #### Pseudo labeling ####
     #### This will be skipped if there is an existing `gpl-training-data.tsv` file under `path_to_generated_data` ####
     if 'gpl-training-data.tsv' in os.listdir(path_to_generated_data):
@@ -96,12 +97,9 @@ def train(
     #### This will be skipped if the checkpoint at the indicated training steps can be found ####
     ckpt_dir = os.path.join(output_dir, str(gpl_steps))
     if not os.path.exists(ckpt_dir) or (os.path.exists(ckpt_dir) and not os.listdir(ckpt_dir)):
-        logger.info('Now training GPL on generated data')
-        #### Provide any HuggingFace model and fine-tune from scratch
-        model_name = base_ckpt
-        word_embedding_model = models.Transformer(model_name, max_seq_length=max_seq_length)
-        pooling_model = models.Pooling(word_embedding_model.get_word_embedding_dimension(), pooling_mode=pooling)
-        model = SentenceTransformer(modules=[word_embedding_model, pooling_model])
+        logger.info('Now doing training on the generated data with the MarginMSE loss')
+        #### It can load checkpoints in both SBERT-format (recommended) and Huggingface-format
+        model: SentenceTransformer = load_sbert(base_ckpt, pooling, max_seq_length)
 
         fpath_gpl_data = os.path.join(path_to_generated_data, 'gpl-training-data.tsv')
         train_dataset = GenerativePseudoLabelingDataset(fpath_gpl_data, gen_queries, corpus)
@@ -123,6 +121,7 @@ def train(
     else:
         logger.info('Trained GPL model found. Now skip training')
 
+
     ### Evaluate the model if required ###
     if do_evaluation:
         logger.info('Doing evaluation for GPL')
@@ -138,10 +137,10 @@ def train(
 
 if __name__ == '__main__':
     parser = argparse.ArgumentParser()
-    parser.add_argument('--path_to_generated_data', required=True, help='Path for/to the generated data. If an empty folder is indicated, query generation and hard-negative mining will be run automatically (the `corpus.jsonl` file in `evaluation_data` will be used!!!); one can also use a BeIR-QGen format data folder to start and skip the query generation.')
-    parser.add_argument('--evaluation_data', required=True, help='Path to the BeIR-format dataset. Please make sure at least `corpus.jsonl` exists under this path, which will be used for GPL training.')
+    parser.add_argument('--path_to_generated_data', required=True, help='Path for/to the generated data. GPL will first check this path for a `corpus.jsonl` file for the (sole) data input of the whole pipeline. If an empty folder is indicated, query generation and hard-negative mining will be run automatically; one can also use a BeIR-QGen format data folder to start and skip the query generation.')
     parser.add_argument('--output_dir', required=True, help='Output path for the GPL model.')
     parser.add_argument('--do_evaluation', action='store_true', default=False, help='Wether to do the evaluation (after training)')
+    parser.add_argument('--evaluation_data', type=str, help='Path to the BeIR-format dataset. This is the next folder GPL goes to for the target corpus if there is no `corpus.jsonl` under `path_to_generated_data`')
     parser.add_argument('--evaluation_output', default='output', help='Path for the evaluation output.')
     parser.add_argument('--qgen_prefix', default='qgen', help='This prefix will appear as part of the (folder/file) names for query-generation results: For example, we will have "gen-qrels/" and "gen-queries.jsonl" by default.')
     parser.add_argument('--base_ckpt', default='distilbert-base-uncased', help='Initialization checkpoint in HF or SBERT format. Meaning-pooling will be used.')
@@ -149,7 +148,7 @@ if __name__ == '__main__':
     parser.add_argument('--cross_encoder', default='cross-encoder/ms-marco-MiniLM-L-6-v2')
     parser.add_argument('--batch_size_gpl', type=int, default=32)
     parser.add_argument('--batch_size_generation', type=int, default=32)
-    parser.add_argument('--pooling', type=str, default='mean', choices=['cls', 'mean', 'max'])
+    parser.add_argument('--pooling', type=str, default=None, choices=['cls', 'mean', 'max'], help="Specifying pooling method for dense retriever if in Huggingface-format. By default (None), it uses mean pooling. If in SBERT-format, there would be the indicated pooling method in its configure file and thus this argument will be ignored. ")
     parser.add_argument('--max_seq_length', type=int, default=350)
     parser.add_argument('--new_size', type=int, default=None, help='Resize the corpus to `new_size` if needed.')
     parser.add_argument('--queries_per_passage', type=int, default=3)
